package main

import (
	"database/sql"
	"encoding/json"
	"fmt"
	"net/http"
	"strconv"
	"time"
)
import _ "github.com/go-sql-driver/mysql"

var Db *sql.DB


type MsgData struct {
	Msg string `json:"msg"`
	Data Post `json:"data"`
}

type Msg struct {
	Code int `json:"code"`
	MsgData MsgData `json:"data"`
}

type Post struct {
	Id int `json:"id"`
	Content string `json:"content"`
	Author string `json:"author"`
	Comments []Comments `json:"comments"`
}

type Comments struct {
	Id int `json:"id"`
	Content string `json:"content"`
	Author string `json:"author"`
	Pid int `json:"pid"`
	Post int `json:"pid"`
}

func EchoErr(err error){
	if (err != nil) {
		panic(err)
	}
}

func setJson(code int, msgs string, data Post) (msg Msg){
	msg = Msg{
		Code:code,
		MsgData:MsgData{
			Msg:msgs,
			Data:data,
		},
	}
	return
}

func formatJson(msg Msg) (data []byte)  {

	data,err := json.MarshalIndent(msg,"","\t")
	EchoErr(err)

	return
}

func init()  {
	//[username[:password]@][protocol[(address)]]/dbname[?param1=value1&...&paramN=valueN]
	var err error
	Db,err = sql.Open("mysql","root:ibaotu@tcp(127.0.0.1:3308)/t")
	EchoErr(err)
}

func GetPost(id string) (post Post){

	err := Db.QueryRow("select id,content,author from posts where id = ?", id).Scan(
		&post.Id, &post.Content, &post.Author)
	EchoErr(err)

	row,err := Db.Query("select id,content,author from  comments where post_id = ?", post.Id)
	EchoErr(err)

	for row.Next(){
		comment := Comments{Post:post.Id}
		row.Scan(&comment.Id, &comment.Content, &comment.Author)
		post.Comments = append(post.Comments, comment)
	}

	return
}


func getHandle(w http.ResponseWriter,r *http.Request) {

	id := r.FormValue("id")

	post := GetPost(id)

	msg := setJson(200, "ok", post)

	postjson,err := json.MarshalIndent(msg ,"", "\t")
	EchoErr(err)

	w.Write(postjson)

}

func (post *Post)create() {
	_,err := Db.Query("insert into posts (content,author) values (?,?)", post.Content, post.Author)
	EchoErr(err)
}

func createHandle(w http.ResponseWriter,r *http.Request) {

	post := Post{
		Content:r.PostFormValue("content"),
		Author:r.PostFormValue("author"),
	}

	post.create()

	json := setJson(200,"写入成功",post)

	w.Write(formatJson(json))

}


func delHandle(w http.ResponseWriter,r *http.Request) {
		id := r.PostFormValue("id")
		re,err :=Db.Exec("delete from posts where id = ?",id)
		EchoErr(err)

		row,_ :=re.RowsAffected()


		s := strconv.FormatInt(row,10)
		fmt.Println(s)

		w.Write(formatJson(setJson(200, "成功删除:"+ s,Post{})))
}


func postHandle(w http.ResponseWriter,r *http.Request)  {
	switch 	r.Method {
	case "GET" :
		getHandle(w,r)
	case "POST" :
		createHandle(w,r)
	case "DELETE" :
		delHandle(w,r)
	}

}
//27.189.231.39 - - [09/Apr/2018:17:21:23 +0800] "GET /Public/index/images/icon_pre.png HTTP/1.1" 200 44668 "http://www.test.com/Public/index/css/global.css" "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/44.0.2403.157 Safari/537.36" "-"
//'$remote_addr - $remote_user [$time_local] "$request" $status $body_bytes_sent "$http_referer" "$http_user_agent" "$http_x_forwarded_for"'
func log(h http.Handler) (HandlerFunc http.HandlerFunc){
	return func(writer http.ResponseWriter, request *http.Request) {
		t :=time.Now().Format("2006-01-02 15:04:05")
		var scheme string

		if (request.TLS == nil){
			scheme = "http://"
		} else {
			scheme = "https://"
		}
		ua := request.Header["User-Agent"]
		fmt.Println(request.RemoteAddr + " - - [" + t + "] " + request.Method + " \"" + request.URL.Path + "\" " +  scheme+request.Host + request.URL.Path + " \"" + ua[0] + "\"")

		h.ServeHTTP(writer,request)
	}
}

func main()  {

	mux := http.NewServeMux()
	mux.HandleFunc("/post", postHandle)

	fmt.Println("启动中 [http://127.0.0.1:3333]")

	err := http.ListenAndServe(":3333",log(mux))

	if (err != nil) {
		fmt.Println("启动失败")
		EchoErr(err)
	}
}

